
$identify("org/mathdox/formulaeditor/modules/logic1/and.js");

$require("org/mathdox/parsing/ParserGenerator.js");
$require("org/mathdox/formulaeditor/parsing/openmath/OpenMathParser.js");
$require("org/mathdox/formulaeditor/parsing/expression/ExpressionParser.js");
$require("org/mathdox/formulaeditor/semantics/MultaryOperation.js");
$require("org/mathdox/formulaeditor/semantics/Keyword.js");

$main(function(){

  var symbol = {
    onscreen : "∧",
    openmath : null, // use default with model:cd and model:name
    mathml   : "<mo>∧</mo>"
  };

  /**
   * Define a semantic tree node that represents logic1.and.
   */
  org.mathdox.formulaeditor.semantics.Logic1And =
    $extend(org.mathdox.formulaeditor.semantics.MultaryOperation, {

      symbol : {

        onscreen : symbol.onscreen,
        openmath : "<OMS cd='logic1' name='and'/>",
        mathml   : symbol.mathml

      },

      precedence : 100

    });
  
  /**
   * Extend the OpenMathParser object with parsing code for logic1.and.
   */
  org.mathdox.formulaeditor.parsing.openmath.OpenMathParser =
    $extend(org.mathdox.formulaeditor.parsing.openmath.OpenMathParser, {

    /**
     * Returns an equality object based on the OpenMath node.
     */
    handleLogic1And : function(node) {

      // parse the children of the OMA
      var children = node.getChildNodes();
      var operands = [];
      for (var i=1; i<children.length; i++) {
        operands.push(this.handle(children.item(i)));
      }

      // construct the corresponding object
      var result = new org.mathdox.formulaeditor.semantics.Logic1And();
      result.initialize.apply(result, operands);
      return result;

    }

  });

  org.mathdox.formulaeditor.parsing.openmath.KeywordList["logic1__and"] = new org.mathdox.formulaeditor.semantics.Keyword("logic1", "and", symbol, "infix");

  /**
   * Extend the ExpressionParser object with parsing code for the equality sign.
   */
  var semantics = org.mathdox.formulaeditor.semantics;
  var pG = new org.mathdox.parsing.ParserGenerator();

  if ( "&&" == "∧" ) {
    // only one expression, same on screen
    org.mathdox.formulaeditor.parsing.expression.ExpressionParser =
      $extend(org.mathdox.formulaeditor.parsing.expression.ExpressionParser, {

      // expression100 = and | super.expression100
      expression100 : function() {
        var parent = arguments.callee.parent;
        pG.alternation(
          pG.rule("logic1and"),
          parent.expression100).apply(this, arguments);
      },

      // logic1and = 
      //    expression100 "&&" expression110
      logic1and :
        pG.transform(
          pG.concatenation(
            pG.rule("expression100"),
            pG.literal("&&"),
            pG.rule("expression110")
          ),
          function(result) {
            return new semantics.Logic1And(result[0], result[2]);
          }
        )

      });
    } else { // allow alternative as displayed on the screen
      org.mathdox.formulaeditor.parsing.expression.ExpressionParser =
        $extend(org.mathdox.formulaeditor.parsing.expression.ExpressionParser, {

        // expression100 = logic1and | 
        //   logic1andalt | super.expression100
        expression100 : function() {
          var parent = arguments.callee.parent;
          pG.alternation(
            pG.rule("logic1and"),
            pG.rule("logic1andalt"),
            parent.expression100).apply(this, arguments);
        },

        // logic1and = 
        //    expression100 "&&" expression110
        logic1and :
          pG.transform(
            pG.concatenation(
              pG.rule("expression100"),
              pG.literal("&&"),
              pG.rule("expression110")
            ),
            function(result) {
              return new semantics.Logic1And(result[0], result[2]);
            }
          ),

        // logic1andalt = 
        //    expression100 "∧" expression110
        logic1andalt :
          pG.transform(
            pG.concatenation(
              pG.rule("expression100"),
              pG.literal("∧"),
              pG.rule("expression110")
            ),
            function(result) {
              return new semantics.Logic1And(result[0], result[2]);
            }
          )

      });
    }

});
